import pytest
import libtcrlm
from libtcrlm import schema
from libtcrlm.schema import exception


def test_cdr1a_sequence(mock_tcr):
    result = mock_tcr.cdr1a_sequence
    expected = "TSGFYG"

    assert result == expected


def test_cdr1b_sequence(mock_tcr):
    result = mock_tcr.cdr1b_sequence
    expected = "SNHLY"

    assert result == expected


def test_cdr2a_sequence(mock_tcr):
    result = mock_tcr.cdr2a_sequence
    expected = "NALDGL"

    assert result == expected


def test_cdr2b_sequence(mock_tcr):
    result = mock_tcr.cdr2b_sequence
    expected = "FYNNEI"

    assert result == expected


def test_junction_a_sequence(mock_tcr):
    result = mock_tcr.junction_a_sequence
    expected = "CATQYF"

    assert result == expected


def test_junction_b_sequence(mock_tcr):
    result = mock_tcr.junction_b_sequence
    expected = "CASQYF"

    assert result == expected


def test_repr(mock_tcr):
    result = repr(mock_tcr)
    expected = "Tra(TRAV1-1*01/CATQYF)/Trb(TRBV2*01/CASQYF)"

    assert result == expected


@pytest.mark.parametrize(
    argnames=("anchor", "comparison", "expected"),
    argvalues=(
        (
            schema.make_tcr_from_components(
                "TRAV1-1*01", "CATQYF", "TRBV2*01", "CASQYF"
            ),
            schema.make_tcr_from_components(
                "TRAV1-1*01", "CATQYF", "TRBV2*01", "CASQYF"
            ),
            True,
        ),
        (
            schema.make_tcr_from_components(
                "TRAV1-1*01", "CATQYF", "TRBV2*01", "CASQYF"
            ),
            schema.make_tcr_from_components(
                "TRAV5*01", "CASSRPLWYF", "TRBV3-1*01", "CASKLAQF"
            ),
            False,
        ),
        (
            schema.make_tcr_from_components(
                "TRAV1-1*01", "CATQYF", "TRBV2*01", "CASQYF"
            ),
            schema.make_tcr_from_components("TRAV1-1*01", "CATQYF", None, None),
            False,
        ),
    ),
)
def test_equality(anchor, comparison, expected):
    result = anchor == comparison
    assert result == expected


def test_allele_imputation():
    tcr = schema.make_tcr_from_components("TRAV1-1", "CATQYF", "TRBV12-2", "CASQYF")
    assert tcr._trav.allele_num == 1
    assert tcr._trbv.allele_num == 2


class TestSetup:
    def test_homosapiens_setup(self):
        tcr = schema.make_tcr_from_components(
            "TRAV1-1*01", "CATQYF", "TRBV2*01", "CASQYF"
        )
        assert tcr.cdr1a_sequence == "TSGFYG"
        assert tcr.cdr2a_sequence == "NALDGL"
        assert tcr.junction_a_sequence == "CATQYF"
        assert tcr.cdr1b_sequence == "SNHLY"
        assert tcr.cdr2b_sequence == "FYNNEI"
        assert tcr.junction_b_sequence == "CASQYF"

        with pytest.raises(exception.BadV):
            tcr = schema.make_tcr_from_components(
                "TRAV1*01", "CATQYF", "TRBV1*01", "CASQYF"
            )

    def test_musmusculus_setup(self, trigger_musmusculus_setup):
        tcr = schema.make_tcr_from_components(
            "TRAV1*01", "CATQYF", "TRBV1*01", "CASQYF"
        )
        assert tcr.cdr1a_sequence == "TSGFNG"
        assert tcr.cdr2a_sequence == "VVLDGL"
        assert tcr.junction_a_sequence == "CATQYF"
        assert tcr.cdr1b_sequence == "NSQYPW"
        assert tcr.cdr2b_sequence == "LRSPGD"
        assert tcr.junction_b_sequence == "CASQYF"

        with pytest.raises(exception.BadV):
            tcr = schema.make_tcr_from_components(
                "TRAV1-1*01", "CATQYF", "TRBV2*01", "CASQYF"
            )


@pytest.fixture
def trigger_musmusculus_setup():
    libtcrlm.setup("musmusculus")
    yield
    libtcrlm.setup("homosapiens")



from libtcrlm import schema
from libtcrlm.bert import Bert
from libtcrlm.token_embedder import SingleChainCdrEmbedder
from libtcrlm.self_attention_stack import SelfAttentionStackWithBuiltins
from libtcrlm.mlm_token_prediction_projector import AminoAcidTokenProjector
from libtcrlm.vector_representation_delegate import (
    AveragePoolVectorRepresentationDelegate,
)
import pandas as pd
from pathlib import Path
import pytest


@pytest.fixture
def toy_bert(toy_bert_d_model):
    token_embedder = SingleChainCdrEmbedder(embedding_dim=toy_bert_d_model)
    self_attention_stack = SelfAttentionStackWithBuiltins(
        num_layers=2, d_model=toy_bert_d_model, nhead=2
    )
    mlm_token_prediction_projector = AminoAcidTokenProjector(d_model=toy_bert_d_model)
    vector_representation_delegate = AveragePoolVectorRepresentationDelegate(
        self_attention_stack=self_attention_stack
    )

    bert = Bert(
        token_embedder=token_embedder,
        self_attention_stack=self_attention_stack,
        mlm_token_prediction_projector=mlm_token_prediction_projector,
        vector_representation_delegate=vector_representation_delegate,
    )

    return bert


@pytest.fixture
def toy_bert_d_model():
    return 4


@pytest.fixture
def mock_tcr():
    return schema.make_tcr_from_components("TRAV1-1*01", "CATQYF", "TRBV2*01", "CASQYF")


@pytest.fixture
def mock_data_path():
    return Path("tests") / "resources" / "mock_data.csv"


@pytest.fixture
def mock_data_df(mock_data_path):
    return pd.read_csv(mock_data_path)



import pytest
import torch
from torch import Tensor

from libtcrlm.self_attention_stack import SelfAttentionStackWithBuiltins
from libtcrlm.vector_representation_delegate import ClsVectorRepresentationDelegate


D_MODEL = 4
BATCH_SIZE = 3
TOKENISED_TCR_LENGTH = 5


@pytest.fixture
def mock_token_embeddings():
    return torch.ones((BATCH_SIZE, TOKENISED_TCR_LENGTH, D_MODEL))


@pytest.fixture
def mock_padding_mask():
    return torch.zeros((BATCH_SIZE, TOKENISED_TCR_LENGTH), dtype=torch.bool)


def test_get_vector_representations_of(mock_token_embeddings, mock_padding_mask):
    self_attention_stack = SelfAttentionStackWithBuiltins(
        num_layers=2, d_model=D_MODEL, nhead=2
    )
    vector_representation_delegate = ClsVectorRepresentationDelegate(
        self_attention_stack
    )

    result = vector_representation_delegate.get_vector_representations_of(
        mock_token_embeddings, mock_padding_mask
    )

    assert type(result) == Tensor
    assert result.dim() == 2
    assert result.size(0) == BATCH_SIZE
    assert result.size(1) == D_MODEL



import pytest
import torch
from torch import Tensor

from libtcrlm.self_attention_stack import SelfAttentionStackWithInitialProjection


EMBEDDING_DIM = 10
D_MODEL = 4
BATCH_SIZE = 3
TOKENISED_TCR_LENGTH = 5


@pytest.fixture
def self_attention_stack_with_initial_projection():
    self_attention_stack_with_builtins = SelfAttentionStackWithInitialProjection(
        num_layers=3, embedding_dim=EMBEDDING_DIM, d_model=D_MODEL, nhead=2
    )
    return self_attention_stack_with_builtins


@pytest.fixture
def mock_token_embeddings():
    return torch.ones((BATCH_SIZE, TOKENISED_TCR_LENGTH, EMBEDDING_DIM))


@pytest.fixture
def mock_padding_mask():
    return torch.zeros((BATCH_SIZE, TOKENISED_TCR_LENGTH), dtype=torch.bool)


def test_forward(
    self_attention_stack_with_initial_projection: SelfAttentionStackWithInitialProjection,
    mock_token_embeddings,
    mock_padding_mask,
):
    result = self_attention_stack_with_initial_projection.forward(
        mock_token_embeddings, mock_padding_mask
    )

    assert type(result) == Tensor
    assert result.dim() == 3
    assert result.size(0) == BATCH_SIZE
    assert result.size(1) == TOKENISED_TCR_LENGTH
    assert result.size(2) == D_MODEL


def test_get_token_embeddings_at_penultimate_layer(
    self_attention_stack_with_initial_projection: SelfAttentionStackWithInitialProjection,
    mock_token_embeddings,
    mock_padding_mask,
):
    result = self_attention_stack_with_initial_projection.get_token_embeddings_at_penultimate_layer(
        mock_token_embeddings, mock_padding_mask
    )

    assert type(result) == Tensor
    assert result.dim() == 3
    assert result.size(0) == BATCH_SIZE
    assert result.size(1) == TOKENISED_TCR_LENGTH
    assert result.size(2) == D_MODEL



import pytest

from libtcrlm import schema


@pytest.mark.parametrize(
    argnames=("anchor", "comparison", "expected"),
    argvalues=(
        (
            schema.make_pmhc_from_components("AAA", "HLA-A*01:01", "B2M"),
            schema.make_pmhc_from_components("AAA", "HLA-A*01:01", "B2M"),
            True,
        ),
        (
            schema.make_pmhc_from_components("AAA", "HLA-A*01:01", "B2M"),
            schema.make_pmhc_from_components("CCC", "HLA-DRA*01:01", "HLA-DRB1*01:01"),
            False,
        ),
        (
            schema.make_pmhc_from_components("AAA", "HLA-A*01:01", "B2M"),
            schema.make_pmhc_from_components("AAA", None, None),
            True,
        ),
        (
            schema.make_pmhc_from_components("CCC", "HLA-DRA*01:01", "HLA-DRB1*01:01"),
            schema.make_pmhc_from_components("CCC", "HLA-DRA", "HLA-DRB1"),
            True,
        ),
        (
            schema.make_pmhc_from_components(None, "HLA-DRA*01:01", "HLA-DRB1*01:01"),
            schema.make_pmhc_from_components(None, "HLA-DRA*01:01", "HLA-DRB1*01:01"),
            False,
        ),
        (
            schema.make_pmhc_from_components("AAA", None, None),
            schema.make_pmhc_from_components("AAA", None, None),
            True,
        ),
    ),
)
def test_equality(anchor, comparison, expected):
    result = anchor == comparison
    assert result == expected


def test_repr():
    pmhc = schema.make_pmhc_from_components("AAA", "HLA-A*01:01", "B2M")
    assert repr(pmhc) == "AAA/HLA-A*01:01/B2M"



from libtcrlm.schema import df_manipulation
import pandas as pd
import pytest


def test_bad_v():
    df = pd.read_csv("tests/resources/bad_trav.csv")
    with pytest.raises(ValueError, match="Bad TRAV symbol at index 2"):
        df_manipulation.generate_tcr_series(df)


def test_bad_junction():
    df = pd.read_csv("tests/resources/bad_cdr3b.csv")
    with pytest.raises(ValueError, match="Bad CDR3B sequence at index 2"):
        df_manipulation.generate_tcr_series(df)
